// Updates: 2004.07.27


import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.event.*;
import java.util.*;


/**
 * A text field that record previous entries and allows to navigate between them
 * using the up and the down arrow keys. This actual version is specifically
 * made to work with the <code>LogViewer</code> class, but can be easily updated
 * to work with any client (using an interface). The constructor gets as first
 * parameter the client that will be called (via the <code>tfChanged</code>
 * method) each time a key is pressed.
 * @author Michel Deriaz
 */
public class JTextFieldX extends JTextField implements ActionListener, KeyListener {
  private LogViewer client;
  private ArrayList cmds;
  private int cmdIndex;

  
  /**
   * Constructs a text field. The specified client (if not null) will be 
   * called (via the <code>tfChanged</code> method) each time a key is pressed.
   * @param client the client class
   * @param columns the size of the text field
   */
  public JTextFieldX(LogViewer client, int columns) {
    super(columns);
    this.client = client;
    cmds = new ArrayList();
    addKeyListener(this);
    addActionListener(this);
  }
  
  
  public void actionPerformed(ActionEvent e) {
    cmds.add(getText());
    cmdIndex = cmds.size();   
    selectAll();
  }  


  public void keyPressed(KeyEvent e) {
    if (e.getKeyCode() == KeyEvent.VK_UP) {
      if (cmdIndex > 0) setText((String)cmds.get(--cmdIndex));
      e.consume();
    }
    if (e.getKeyCode() == KeyEvent.VK_DOWN) {
      if (cmdIndex < cmds.size()-1) setText((String)cmds.get(++cmdIndex));
      e.consume();
    }
  }

  
  public void keyReleased(KeyEvent e) {
    if (client != null) client.tfChanged(this);
  }
  
  
  public void keyTyped(KeyEvent e) {}   
}