// Updates: 2004.07.27


import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.event.*;
import java.util.*;
import java.io.*;
import java.awt.datatransfer.*;
import java.awt.dnd.*;
import java.util.regex.*;


public class LogViewer extends JFrame implements ActionListener, DropTargetListener {
  private String text = null;
  private StyledTextZone stz;
  private JCheckBox cbMsg, cbEnter, cbExit;
  private JTextFieldX tf;
  private JButton butSearch;
  private JCheckBox cbLive;
  private JButton butOpen;
 
  
  public LogViewer() {
    super("Log Viewer");
    JPanel p = new JPanel(new BorderLayout());
    p.setBorder(BorderFactory.createEmptyBorder(5, 5, 5, 5));
    
    stz = new StyledTextZone("Log file", 0);
    stz.setEditable(false);
    stz.setFont(-1, "Monospaced");
    p.add(stz, BorderLayout.CENTER);
    
    JPanel panSouth = new JPanel(new GridLayout(2, 1));

    JPanel panCmd = new JPanel();
    cbMsg = new JCheckBox("msg", true);
    cbMsg.addActionListener(this);
    panCmd.add(cbMsg);
    cbEnter = new JCheckBox("enter", true);
    cbEnter.addActionListener(this);
    panCmd.add(cbEnter);
    cbExit = new JCheckBox("exit", true);
    cbExit.addActionListener(this);
    panCmd.add(cbExit);
    panCmd.add(new JLabel("          "));
    butOpen = new JButton("Open");
    butOpen.addActionListener(this);
    new DropTarget(butOpen, this);
    panCmd.add(butOpen);    
    panSouth.add(panCmd);
    
    JPanel panRegex = new JPanel();
    panRegex.add(new JLabel("Search (regex): "));
    tf = new JTextFieldX(this, 42);
    panRegex.add(tf);
    butSearch = new JButton("Search");
    butSearch.setEnabled(false);
    butSearch.addActionListener(this);
    panRegex.add(butSearch);
    cbLive = new JCheckBox("Live eval.", true);
    cbLive.addActionListener(this);
    panRegex.add(cbLive);
    panSouth.add(panRegex);
    
    p.add(panSouth, BorderLayout.SOUTH);
    
    getContentPane().add(p);
    setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);    
    setSize(800, 600);
    setLocationRelativeTo(null);
    try {
      UIManager.setLookAndFeel("com.incors.plaf.kunststoff.KunststoffLookAndFeel");
    }
    catch (Exception e) {System.out.println(e);}
    SwingUtilities.updateComponentTreeUI(this);
    show();
    help();
  }


  public static void main(String[] args) {
    new LogViewer();
  }
  
  
  public void actionPerformed(ActionEvent e) {
    if (e.getSource().equals(butOpen)) open();
    if (e.getSource().equals(cbLive)) butSearch.setEnabled(!cbLive.isSelected());
    searchPatterns();
  }
  
  public void drop(DropTargetDropEvent event) {
    event.acceptDrop(DnDConstants.ACTION_COPY);
    Transferable transferable = event.getTransferable();
    DataFlavor[] flavors = transferable.getTransferDataFlavors();
    for (int i = 0; i < flavors.length; i++) { 
      DataFlavor df = flavors[i];
      try {
        if (df.equals(DataFlavor.javaFileListFlavor)) {
          java.util.List fileList = (java.util.List)transferable.getTransferData(df);
          Iterator iterator = fileList.iterator();
          while (iterator.hasNext()) {
            String filename = ((File)iterator.next()).toString();
            String newText = readFile(filename);
            if (newText != null) {            
              setTitle("Log Viewer   -   " + filename);
              text = newText;
              searchPatterns();
            }
          }
        }
      }
      catch(Exception e) { 
        System.out.println(e);
      }
    }
    event.dropComplete(true);
  }


  protected void tfChanged(JTextField src) {
    if (cbLive.isSelected()) searchPatterns();
  }


  public void dragEnter(DropTargetDragEvent event) {}
  public void dragExit(DropTargetEvent event) {}
  public void dragOver(DropTargetDragEvent event) {}
  public void dropActionChanged(DropTargetDragEvent event) {}  
    
  
  private void displayText() {
    stz.setText("");
    if (text == null) return;
    String[] tockens = text.split("\n");
    for (int i = 0; i < tockens.length; i++) {
      String s = tockens[i];
      if (cbMsg.isSelected() && s.startsWith("*msg")) stz.println(s, StyledTextZone.BLUE);
      if (cbEnter.isSelected() && s.startsWith("*enter")) stz.println(s, StyledTextZone.RED);
      if (cbExit.isSelected() && s.startsWith("*exit")) stz.println(s, StyledTextZone.GREEN);
    }
  }  
    
  
  private void open() {
    JFileChooser chooser = new JFileChooser(new File("."));
    chooser.setFileFilter(new javax.swing.filechooser.FileFilter() {
      public boolean accept(File f) {
        return true;
      }
      public String getDescription() {
        return "Log files";
      }
    });    
    int returnVal = chooser.showOpenDialog(this);
    if (returnVal == JFileChooser.APPROVE_OPTION) {
      String filename = chooser.getSelectedFile().getPath();
      String newText = readFile(filename);
      if (newText != null) {
        setTitle("Log Viewer   -   " + filename);
        text = newText;
      }
      else JOptionPane.showMessageDialog(this, "File not found", "Error", JOptionPane.ERROR_MESSAGE);    
    }
  }
  
  
  private String readFile(String filename) {
    File f = new File(filename);
    int size = (int)f.length();
    char cbuf[] = new char[size];
    try {
      FileReader fr = new FileReader(filename);    
      fr.read(cbuf);
      fr.close();
    }
    catch(IOException ioe) {
      return null;
    }
    return new String(cbuf);
  } 
  
  
  private void searchPatterns() {
    displayText();
    stz.saveScrollBarPos();
    String regex = tf.getText();
    if (regex.equals("")) {
      stz.restoreScrollBarPos();
      return;
    }
    try {
      Pattern p = Pattern.compile(regex);
      Matcher m = p.matcher(stz.getText());
      while (m.find()) {    
        stz.setStyle(StyledTextZone.GREENBACK, m.start(), m.end());
      }
    }
    catch(PatternSyntaxException e) {}
    stz.restoreScrollBarPos();    
  }
  
  
  private void help() {
    String helpText = "\n\n- You can also open a logging file with a drag-and-drop operation over the Open button."
        + "\n\n- Use regular expressions to search specific patterns in the file. More information about regular"
        + "\nexpressions can be found at http://cui.unige.ch/~deriazm/Softs/RegexSR/"
        + "\n\n- To record a regular expression, press the enter key; to restore previous expressions, use the up"
        + "\nand down arrows.";
    stz.setText(helpText);    
  }
}